#!/usr/local/bin/php
<?php

/*
 * Munge logs from rblsmtpd (qmail) and put them into an rrd file
 * for handling by rrdtool.
 *
 * Gaby Vanhegan <gaby@vanhegan.net> 2005-07-19
 *
 * This script requires the script tai64n2unix to work with the default
 * rblsmtpd timestamps.  You can either install this application to convert
 * the timestamps, or you can tweak the way multilog works so that it logs
 * in a different format that does not require conversion.
 *
 * Sample log line:
 * @4000000041b08be60ef96984 rblsmtpd: 195.42.160.18 pid 15201: 451 http://dsbl.org/listing?195.42.160.18
 *
 */

// What is the base folder that this script is working in?
$BASE		= "/home/gaby/prel/rrd";

// What folder are the RRD files found in?
$BASE_RRD	= "/var/symon/rrds/localhost";

// What is the source file for log entries?
$LOG		= "/home/gaby/rrd-logs/rblsmtpd.log";

// What is the temporary file we work with?
$LOG_TMP	= tempnam( "/tmp", "rrd-rblsmtpd" );

// What is the cache file for entries that we're logging later?
$CACHE		= "$BASE/rblsmtpd.cache";

/* --- === Nothing to edit below this line === --- */

// No log?  Nothing to do.
if ( !( file_exists( $LOG ) ) ) { exit; }

$lists		= array();

// Log some output text please
function log_msg ( $input ) {
	$msg	= trim( $input );
	$lines	= explode( "\n", $msg );
	foreach ( $lines as $line ) {
		$when	= date( "Y-m-d H:i:s" );
		print "[$when] $line\n";
		}
	}

// Convert a tai64n to a unix timestamp 
function tai64n2unix ( $tai ) {

	// Convert TAI64N to local time
	$result	= shell_exec( "echo '$tai' | /usr/local/bin/tai64nlocal" );
	$result		= trim( $result );

	// The result is 2004-12-01 19:54:32.159264500
	// Chop off the microseconds
	list( $date, $chaff )	= explode( ".", $result );

	// Convert to epoch time and return
	$return		= strtotime( $date );
	return $return;
	}

// Move the log file out of active duty
rename( $LOG, $LOG_TMP );

// Append the logfile to end of the cache
$fh				= fopen( $LOG_TMP, "r" );
$cache			= fopen( $CACHE,   "a" );

while( $line = fgets( $fh ) ) { fputs( $cache, $line ); }

// Close both files
fclose( $cache );
fclose( $fh    );

// Clear the temp logfile
unlink( $LOG_TMP );

// Now open up our composite file
$fh				= fopen( $CACHE, "r" );

// What is the time the script was run?
$right_now		= time();

// What is the cut off time for entries to process?  At least
// half an hour in the past, to the nearest half hour.
$cut_off_time	= ( $right_now - 1800 );
$cut_off_time	= ( $cut_off_time - ( $cut_off_time % 1800 ) );

// Lines that we should cache instead of processing now
$new_cache		= array();

// Now going through our log data...
while( $line = fgets( $fh ) ) {

	$line	= trim( $line );
	$hit	= false;

	// Single up the whitespace in the line
	$hit	= preg_match( "/^(@.+) rblsmtpd: .+http:\/\/(.+)\.[com|net|org].+$/", $line, $matches );

	// If it matches a line
	if ( $hit ) { 

		// Normalise the time to the nearest half hour (1800 seconds)
		$the_time	= tai64n2unix( $matches[1] );
		if ( $the_time > $right_now ) { $the_time = $right_now; }
		$the_time	= ( $the_time - ( $the_time % 1800 ) );

		// If the time of this entry is after the cut off time, we'll
		// add this entry to the cache for processing next time
		if ( $the_time >= $cut_off_time ) { 
			array_push( $new_cache, $line );
			//print "Cached   : " . date( "d/m/Y H:i:s", $the_time ) . " - $line";
			}
		
		// Otherwise, we'll process it now
		else {

			// Which list?
			$list		= strtolower( $matches[2] );
			$list		= str_replace( "www.", "", $list );
	
			$lists[ $list ][ $the_time ][ "blocks" ]	+= 1;
			$lists[ "all" ][ $the_time ][ "blocks" ]	+= 1;
			$lists[ "all" ][ $the_time ][ "time" ]		+= 0;
			}
		}
	}

fclose( $fh );

reset( $new_cache );

// Now write out the cache file again
$cache	= fopen( $CACHE, "w" );
foreach( $new_cache as $line ) { fputs( $cache, $line ); }
fclose( $cache );

foreach ( $lists as $list => $data ) {

	$rrd_file	= "rblsmtpd_" . $list . ".rrd";
	if ( $list == "all" ) { $rrd_file = "rblsmtpd.rrd"; }

	$rrd_file		= "$BASE_RRD/$rrd_file";
	$command_base	= "/usr/local/bin/rrdtool update $rrd_file";
	$command		= $command_base;
	$count			= 1;

	if ( file_exists( $rrd_file ) ) {

		// Sort out data in ascending date please
		ksort( $data );

		foreach ( $data as $time => $values ) {

			$count++;

			$blocks		= $values[ "blocks" ];
			$wait		= $values[ "time" ];

			if ( ( $count % 30 ) < 1 ) {
				//log_msg( $command );
				print trim( shell_exec( $command ) );
				$command	= $command_base;
				}

			if ( ( $blocks > 0 ) or ( $wait > 0 ) ) {
				$command	.= " $time:$blocks:$wait";
				}
			}

		// If there's data to log
		if ( $command != $command_base ) {
			//log_msg( $command );
			print trim( shell_exec( $command ) );
			}
		}
	else {
		log_msg( "$rrd_file does not exist!" );
		}
	}

?>